( cd usr/lib ; rm -rf libSDL-1.2.so.0 )
( cd usr/lib ; ln -sf libSDL-1.2.so.0.7.2 libSDL-1.2.so.0 )
( cd usr/lib ; rm -rf libSDL.so )
( cd usr/lib ; ln -sf libSDL-1.2.so.0.7.2 libSDL.so )
( cd usr/lib ; rm -rf libSDL_image-1.2.so.0 )
( cd usr/lib ; ln -sf libSDL_image-1.2.so.0.1.3 libSDL_image-1.2.so.0 )
( cd usr/lib ; rm -rf libSDL_image.so )
( cd usr/lib ; ln -sf libSDL_image-1.2.so.0.1.3 libSDL_image.so )
( cd usr/lib ; rm -rf libSDL_mixer-1.2.so.0 )
( cd usr/lib ; ln -sf libSDL_mixer-1.2.so.0.2.4 libSDL_mixer-1.2.so.0 )
( cd usr/lib ; rm -rf libSDL_mixer.so )
( cd usr/lib ; ln -sf libSDL_mixer-1.2.so.0.2.4 libSDL_mixer.so )
( cd usr/lib ; rm -rf libSDL_net-1.2.so.0 )
( cd usr/lib ; ln -sf libSDL_net-1.2.so.0.0.5 libSDL_net-1.2.so.0 )
( cd usr/lib ; rm -rf libSDL_net.so )
( cd usr/lib ; ln -sf libSDL_net-1.2.so.0.0.5 libSDL_net.so )
( cd usr/lib ; rm -rf libSDL_ttf-2.0.so.0 )
( cd usr/lib ; ln -sf libSDL_ttf-2.0.so.0.6.1 libSDL_ttf-2.0.so.0 )
( cd usr/lib ; rm -rf libSDL_ttf.so )
( cd usr/lib ; ln -sf libSDL_ttf-2.0.so.0.6.1 libSDL_ttf.so )
